# Memory Benchmark

The `MemoryBenchmark.ino` was compiled with each `FEATURE_*` and the flash
memory and static RAM sizes were recorded. The `FEATURE_BASELINE` selection is
the baseline, and its memory usage  numbers are subtracted from the subsequent
`FEATURE_*` memory usage.

**Version**: AceTime v2.2.0

**DO NOT EDIT**: This file was auto-generated using `make README.md`.

## How to Regenerate

To regenerate this README.md:

```
$ make clean_benchmarks
$ make benchmarks
$ make README.md
```

The `make benchmarks` target uses `collect.sh` script which calls `auniter.sh`
(https://github.com/bxparks/AUniter) to invoke the Arduino IDE programmatically.
It produces a `*.txt` file with the flash and ram usage information (e.g.
`nano.txt`). It now takes about 16 minutes to generate the `*.txt` files on my
quad-core Intel Core i7-3840QM CPU @ 2.80GHz laptop.

The `make README.md` command calls the `generated_readme.py` Python script which
generates this `README.md` file. The ASCII tables below are generated by the
`generate_table.awk` script, which takes each `*.txt` file and converts it to an
ASCII table.

## Library Size Changes

**v1.3:**
* The `BasicZoneManager` and `ExtendedZoneManager` classes were unified under a
  new parent interface `ZoneManager`. This seems to have caused the flash size
  to increase by around 1200 bytes on the AVR processors (Nano, Pro Micro),
  about 500 bytes on a SAMD, about 800 bytes on a ESP8266, 100 bytes on a ESP32,
  and 1400 bytes on a Teensy 3.2. The 8-bit processors suffer the most flash
  size increase proportional to their limited 32kB limit.
* Adding the `ZoneManager` interface simplifies a lot of the complexity with
  saving and restoring time zones using the `TimeZoneData` object, and I think
  it is worth the extra cost of flash size. The mitigating factor is that
  applications targetted towards 8-bit processors will normally have fixed
  number of timezones at compile time, so they can avoid using a `ZoneManager`,
  and avoid this penalty in flash size.

**v1.4.1+:**
* Removed the `ZoneInfo::transitionBufSize` field from the `ZoneInfo` struct,
  which saves 1 byte on 8-bit processors (none on 32-bit processors due to
  4-byte alignment). We save 266 bytes for `BasicZoneManager` and 386 bytes for
  `ExtendedZoneManager` when all the zones are loaded into the zone registry.
* Incorporated zoneName compression causes flash/ram usage to increase by
  ~250/120 bytes when using only 1-2 zones, but *decreases* flash consumption by
  1200-2400 bytes when all the zones are loaded into the `ZoneManager`.

**v1.5+:**
* Changing `ZoneProcessorCache::getType()` from a `virtual` to a non-virtual
  method saves 250-350 bytes of flash memory when using a `BasicZoneManager` or
  an `ExtendedZoneManager` on an 8-bit AVR processor. Unexpectedly, the flash
  memory consumption *increases* slightly (~0-50 bytes) for some ARM processors
  and the ESP32. Since those processors have far more flash memory, this seems
  like a good tradeoff.
* Changing `BasicZoneProcessor` and `ExtendedZoneProcessor` to be subclasses of
  the templatized `BasicZoneProcessorTemplate` and
  `ExtendedZoneProcessorTemplate` classes causes reduction of flash consumption
  by 250-400 bytes for 32-bit processors. Don't know why. (Very little
  difference for 8-bit AVR).
* Adding a `BrokerFactory` layer of indirection (to support more complex
  ZoneProcessors and Brokers) causes flash memory to go up by 100-200 bytes.

**v1.6:**
* Added support for `LinkRegistry` to `BasicZoneManager` and
  `ExtendedZoneManager`. This increases the flash memory usage by 150-500 bytes
  when using one of these classes due to the code required by `LinkRegistrar`. This extra cost is incurred even if the `LinkRegistry` is set to 0 elements. Each `LinkEntry` consumes 8 bytes (2 x `uint32_t`). So a
  `zonedb::kLinkRegistry` with 183 elements uses 1464 extra bytes of flash; a
  `zonedbx::kLinkRegistry` with 207 elements uses 1656 extra bytes.

**v1.7:**
* The virtual destructor on the `Clock` base class removed. This reduced the
  flash usage by 618 bytes on AVR processors , 328 bytes on the SAMD21, but only
  50-60 bytes on other 32-bit processors.
* The various `printShortNameTo()` or `printShortTo()` methods changed to
  replace the underscore in the zone names (e.g. `Los_Angeles`) with spaces
  (e.g. `Los Angeles`) to be more human friendly. This made little difference in
  the flash memory consumption, except on the ESP32 where it increased by
  200-300 bytes.

**v1.7.2**
* The `SystemClock::clockMillis()` is now non-virtual, using compile-time
  polymorphism through C++ template, and incorporating the same techniques from
  AceRoutine v1.3. Saves about 20-40 bytes of flash.

**v1.7.5:**
* `ExtendedZoneProcessor.compareTransitionToMatch()` was modified to
  detect an exact equality between a `Transition` and its `MatchingEra` if any
  of the 3 time stamp versions ('w', 's', 'u') are equal. Adds about 120-150
  bytes of flash on 8-bit and 32-bit processors. But removing
  `originalTransitionTime` from `Transition` decreases flash usage by about 20
  bytes.
* Upgrade ESP8266 Core from 2.7.4 to 3.0.2. Flash consumption increases by
  3-5 kB across the board.
* Upgrade Teensyduino from 1.54 to 1.55. Add memory consumed by `malloc()` and
  `free()` when using classes with virtual methods into baseline
  MemoryBenchmark, reducing the actual memory usage of various features by
  ~3kB.

**v1.8.0:**
* Move Clock and SystemClock benchmarks into AceTimeClock v1.0.0.
* Extract thin links from BasicZoneManager and ExtendedZoneManager into
  new BasicLinkManager and ExtendedLinkManager classes.
    * Saves 200-500 bytes of flash for BasicZoneManager and ExtendedZoneManager.
    * Applications can decide whether to use thin links through the LinkManager
      (~2000 flash bytes for AVR) or use fat links through the
      `kZoneAndLinkRegistry` (~5000 flash bytes for AVR).
* Create various test objects as global variables instead of stack variables
  to get a more accurate measurement of their static memory consumption.

**v1.9.0:**
* Reduce flash usage of `BasicZoneManager` and `ExtendedZoneManager` by
  1100-1300 bytes on AVR processors:
    * Extract `BasicZoneProcessorCache` and `ExtendedZoneProcessorCache` out
      of `BasicZoneManager` and `ExtendedZoneManager`, making them
      non-templatized.
    * Remove all `virtual` methods from `ZoneManager`, making the ZoneManager
      hierarchy non-polymorphic.
    * Looks like I am reverting some of the changes made in v1.3 when I created
      the `ZoneManager` interface.
* Reduce flash usage of `BasicLinkManager` and `ExtendedLinkManager` by
  68 bytes on AVR processors by removing pure `virtual` methods on `LinkManager`
  base class.
* Increase flash usage by 34 bytes on AVR processors due to slight refactoring
  of `getHighWater()` with `getAllocSize()`. Only 4-8 bytes increase on 32-bit
  processors.

**v1.10.0:**
* Remove support for SAMD21 boards.
    * Arduino IDE 1.8.19 with SparkFun SAMD 1.8.6 can no longer upload binaries
      to these boards. Something about bossac 1.7.0 not found.
* Add memory consumption benchmarks for `ZoneSorterByName` and
  `ZoneSorterByOffsetAndName` for `BasicZoneManager` and `ExtendedZoneManager`.
    * AVR: 180-530 bytes of flash
    * 32-bit: 120-600 bytes of flash
* Upgrade tool chain:
    * Arduino IDE from 1.8.13 to 1.8.19
    * Arduino AVR from 1.8.3 to 1.8.4
    * STM32duino from 2.0.0 to 2.2.0
    * ESP32 from 1.0.6 to 2.0.2
    * Teensyduino from 1.55 to 1.56
* Add support for `fold` parameter in `LocalDateTime`, `OffsetDateTime`,
  `ZonedDateTime`, and `ExtendedZoneProcessor`. Increases flash usage:
    * AVR:
        * ~600 bytes, in `ExtendedZoneProcessor` for additional search logic,
        * ~150 bytes, `BasicZoneProcessor`, to carry along the `fold` parameter
    * most 32-bit: 400-600 bytes
    * Teensy: 1300 bytes (no idea why)

**v1.11.0**
* Upgrade ZoneInfo database so that Links are symbolic links to Zones, instead
  of hard links to Zones.
    * Allows Links to know whether they are links.
    * Allows extraction of the zoneId and zoneNames of the target Zone.
    * AVR: Increases flash consumption by ~270 bytes.
    * STM32: Increases flash by 120-150 bytes.
    * ESP8266: Increases flash by 250-300 bytes.
    * ESP32: Increases flash by ~190 bytes.
    * Teensy 3.2: Increase flash by 450-1300 bytes.

**v1.11.1**
* Change `ZoneInfoBroker::targetZoneInfo()` method to return a `ZoneInfoBroker`
  instead of raw `ZoneInfo*` pointer.
    * Increases flash usage by 4-16 bytes for the most part.

**v1.11.5**
* Upgrade tool chain
    * Arduino CLI from 0.20.2 to 0.27.1
    * Arduino AVR Core from 1.8.4 to 1.8.5
    * STM32duino from 2.2.0 to 2.3.0
    * ESP32 Core from 2.0.2 to 2.0.5
    * Teensyduino from 1.56 to 1.57
* Upgrade TZDB from 2022b to 2022d

**v2.0**
* Use `int16_t` year fields.
* Implement adjustable epoch year.
* Upgrade to TZDB 2022f.
* AVR:
    * BasicZoneManager increases ~200 bytes
    * ExtendedZoneManager increases ~500 bytes
    * `zonedb` increases ~1.5 kiB
    * `zonedbx` increases ~3 kiB
* ESP8266
    * BasicZoneManager increases ~50 bytes
    * ExtendedZoneManager increases ~150 bytes
    * `zonedb` increases ~300 bytes
    * `zonedbx` increases ~1.5 kiB

**v2.0.1**
* Upgrade to TZDB 2022g.
* Incorporate `ZonedExtra` to replace various `TimeZone` methods.

**v2.1.0**
* Remove `LinkManager` and `LinkRegistry`.
* Unify links, adding an additional `targetInfo` field in `ZoneInfo`.
    * Increases flash by ~1kB on 8-bit and ~2kB on 32-bit for ~600 zones.

**v2.1.1+**
* Simplify ZoneRule.letter handling to use ZoneRule.letterIndex for all letters,
  not just ones over 1 character long. On 8-bit AVR:
    * BasicZoneProcessor
        * Increases flash consumption for 1-2 zones by ~200 bytes.
        * No change for the full TZ database.
    * ExtendedZoneProcessor
        * No change for 1-2 zones.
        * Decreases flash consumption by ~300 bytes for full TZ database.
**v2.2.0**
* Upgrade tool chain
    * Arduino AVR from 1.8.5 to 1.8.6
    * STM32duino from 2.3.0 to 2.4.0
    * ESP8266 from 3.0.2 to 3.1.2 failed, reverted back to 3.0.2
    * ESP32 from 2.0.5 to 2.0.7
* Add support for Seeed XIAO SAMD21
    * Seeeduino 1.8.3
* Upgrade to TZDB 2023b

# Legend

* [1] Delta flash and ram consumption for `ZoneSorterByName` and
  `ZoneSorterByOffsetAndName` are calculated by subtracting the
  `BasicZoneManager (1 zone)` numbers, to isolate the memory consumption
  of just the sorter classes.
* [2] Delta flash and ram consumption for `ZoneSorterByName` and
  `ZoneSorterByOffsetAndName` are calculated by subtracting the
  `ExtendedZoneManager (1 zone)` numbers, to isolate the memory
  consumption of just the sorter classes.

## Arduino Nano

* 16MHz ATmega328P
* Arduino IDE 1.8.19, Arduino CLI 0.31.0
* Arduino AVR Boards 1.8.6

```
+---------------------------------------------------------------------+
| Functionality                          |  flash/  ram |       delta |
|----------------------------------------+--------------+-------------|
| baseline                               |    474/   11 |     0/    0 |
|----------------------------------------+--------------+-------------|
| LocalDateTime                          |   1108/   21 |   634/   10 |
| ZonedDateTime                          |   1440/   28 |   966/   17 |
| Manual ZoneManager                     |   1398/   13 |   924/    2 |
|----------------------------------------+--------------+-------------|
| Basic TimeZone (1 zone)                |   6870/  355 |  6396/  344 |
| Basic TimeZone (2 zones)               |   7336/  503 |  6862/  492 |
| BasicZoneManager (1 zone)              |   7074/  366 |  6600/  355 |
| BasicZoneManager (all zones)           |  19764/  744 | 19290/  733 |
| BasicZoneManager (all zones+links)     |  25116/  744 | 24642/  733 |
|----------------------------------------+--------------+-------------|
| Basic ZoneSorterByName [1]             |   7690/  368 |   616/    2 |
| Basic ZoneSorterByOffsetAndName [1]    |   7862/  368 |   788/    2 |
|----------------------------------------+--------------+-------------|
| Extended TimeZone (1 zone)             |   9992/  742 |  9518/  731 |
| Extended TimeZone (2 zones)            |  10508/ 1207 | 10034/ 1196 |
| ExtendedZoneManager (1 zone)           |  10168/  748 |  9694/  737 |
| ExtendedZoneManager (all zones)        |  34416/ 1208 | 33942/ 1197 |
| ExtendedZoneManager (all zones+links)  |  40420/ 1208 | 39946/ 1197 |
|----------------------------------------+--------------+-------------|
| Extended ZoneSorterByName [2]          |  10780/  750 |   612/    2 |
| Extended ZoneSorterByOffsetAndName [2] |  10958/  750 |   790/    2 |
+---------------------------------------------------------------------+

```

## Sparkfun Pro Micro

* 16 MHz ATmega32U4
* Arduino IDE 1.8.19, Arduino CLI 0.31.0
* SparkFun AVR Boards 1.1.13

```
+---------------------------------------------------------------------+
| Functionality                          |  flash/  ram |       delta |
|----------------------------------------+--------------+-------------|
| baseline                               |   3470/  153 |     0/    0 |
|----------------------------------------+--------------+-------------|
| LocalDateTime                          |   4080/  161 |   610/    8 |
| ZonedDateTime                          |   4412/  168 |   942/   15 |
| Manual ZoneManager                     |   4392/  153 |   922/    0 |
|----------------------------------------+--------------+-------------|
| Basic TimeZone (1 zone)                |   9842/  495 |  6372/  342 |
| Basic TimeZone (2 zones)               |  10290/  641 |  6820/  488 |
| BasicZoneManager (1 zone)              |  10046/  506 |  6576/  353 |
| BasicZoneManager (all zones)           |  22734/  882 | 19264/  729 |
| BasicZoneManager (all zones+links)     |  28086/  882 | 24616/  729 |
|----------------------------------------+--------------+-------------|
| Basic ZoneSorterByName [1]             |  10662/  508 |   616/    2 |
| Basic ZoneSorterByOffsetAndName [1]    |  10834/  508 |   788/    2 |
|----------------------------------------+--------------+-------------|
| Extended TimeZone (1 zone)             |  12946/  880 |  9476/  727 |
| Extended TimeZone (2 zones)            |  13464/ 1347 |  9994/ 1194 |
| ExtendedZoneManager (1 zone)           |  13122/  886 |  9652/  733 |
| ExtendedZoneManager (all zones)        |  37388/ 1348 | 33918/ 1195 |
| ExtendedZoneManager (all zones+links)  |  43392/ 1348 | 39922/ 1195 |
|----------------------------------------+--------------+-------------|
| Extended ZoneSorterByName [2]          |  13734/  888 |   612/    2 |
| Extended ZoneSorterByOffsetAndName [2] |  13912/  888 |   790/    2 |
+---------------------------------------------------------------------+

```

## Seeed Studio XIAO SAMD21

* SAMD21, 48 MHz ARM Cortex-M0+
* Arduino IDE 1.8.19, Arduino CLI 0.31.1
* Seeeduino SAMD Boards 1.8.3

```
+---------------------------------------------------------------------+
| Functionality                          |  flash/  ram |       delta |
|----------------------------------------+--------------+-------------|
| baseline                               |  34068/    0 |     0/    0 |
|----------------------------------------+--------------+-------------|
| LocalDateTime                          |  34372/    0 |   304/    0 |
| ZonedDateTime                          |  35140/    0 |  1072/    0 |
| Manual ZoneManager                     |  35124/    0 |  1056/    0 |
|----------------------------------------+--------------+-------------|
| Basic TimeZone (1 zone)                |  38636/    0 |  4568/    0 |
| Basic TimeZone (2 zones)               |  38972/    0 |  4904/    0 |
| BasicZoneManager (1 zone)              |  38740/    0 |  4672/    0 |
| BasicZoneManager (all zones)           |  55580/    0 | 21512/    0 |
| BasicZoneManager (all zones+links)     |  63788/    0 | 29720/    0 |
|----------------------------------------+--------------+-------------|
| Basic ZoneSorterByName [1]             |  39212/    0 |   472/    0 |
| Basic ZoneSorterByOffsetAndName [1]    |  39260/    0 |   520/    0 |
|----------------------------------------+--------------+-------------|
| Extended TimeZone (1 zone)             |  40684/    0 |  6616/    0 |
| Extended TimeZone (2 zones)            |  41028/    0 |  6960/    0 |
| ExtendedZoneManager (1 zone)           |  40788/    0 |  6720/    0 |
| ExtendedZoneManager (all zones)        |  72740/    0 | 38672/    0 |
| ExtendedZoneManager (all zones+links)  |  81924/    0 | 47856/    0 |
|----------------------------------------+--------------+-------------|
| Extended ZoneSorterByName [2]          |  41260/    0 |   472/    0 |
| Extended ZoneSorterByOffsetAndName [2] |  41308/    0 |   520/    0 |
+---------------------------------------------------------------------+

```

## STM32 Blue Pill

* STM32F103C8, 72 MHz ARM Cortex-M3
* Arduino IDE 1.8.19, Arduino CLI 0.31.0
* STM32duino 2.4.0

```
+---------------------------------------------------------------------+
| Functionality                          |  flash/  ram |       delta |
|----------------------------------------+--------------+-------------|
| baseline                               |  21392/ 3556 |     0/    0 |
|----------------------------------------+--------------+-------------|
| LocalDateTime                          |  21740/ 3572 |   348/   16 |
| ZonedDateTime                          |  22480/ 3588 |  1088/   32 |
| Manual ZoneManager                     |  22468/ 3564 |  1076/    8 |
|----------------------------------------+--------------+-------------|
| Basic TimeZone (1 zone)                |  26032/ 3768 |  4640/  212 |
| Basic TimeZone (2 zones)               |  26384/ 3972 |  4992/  416 |
| BasicZoneManager (1 zone)              |  26152/ 3788 |  4760/  232 |
| BasicZoneManager (all zones)           |  42996/ 3788 | 21604/  232 |
| BasicZoneManager (all zones+links)     |  51204/ 3788 | 29812/  232 |
|----------------------------------------+--------------+-------------|
| Basic ZoneSorterByName [1]             |  26612/ 3792 |   460/    4 |
| Basic ZoneSorterByOffsetAndName [1]    |  26660/ 3792 |   508/    4 |
|----------------------------------------+--------------+-------------|
| Extended TimeZone (1 zone)             |  27844/ 4180 |  6452/  624 |
| Extended TimeZone (2 zones)            |  28200/ 4796 |  6808/ 1240 |
| ExtendedZoneManager (1 zone)           |  27960/ 4188 |  6568/  632 |
| ExtendedZoneManager (all zones)        |  59908/ 4188 | 38516/  632 |
| ExtendedZoneManager (all zones+links)  |  69096/ 4188 | 47704/  632 |
|----------------------------------------+--------------+-------------|
| Extended ZoneSorterByName [2]          |  28428/ 4192 |   468/    4 |
| Extended ZoneSorterByOffsetAndName [2] |  28476/ 4192 |   516/    4 |
+---------------------------------------------------------------------+

```

An entry of `-1` indicates that the memory usage exceeded the maximum of the
microcontroller and the compiler did not generate the desired information.

## ESP8266

* NodeMCU 1.0, 80MHz ESP8266
* Arduino IDE 1.8.19, Arduino CLI 0.31.0
* ESP8266 Boards 3.0.2

```
+---------------------------------------------------------------------+
| Functionality                          |  flash/  ram |       delta |
|----------------------------------------+--------------+-------------|
| baseline                               | 260089/27892 |     0/    0 |
|----------------------------------------+--------------+-------------|
| LocalDateTime                          | 260613/27912 |   524/   20 |
| ZonedDateTime                          | 261733/27928 |  1644/   36 |
| Manual ZoneManager                     | 261713/27900 |  1624/    8 |
|----------------------------------------+--------------+-------------|
| Basic TimeZone (1 zone)                | 266965/28704 |  6876/  812 |
| Basic TimeZone (2 zones)               | 267461/28904 |  7372/ 1012 |
| BasicZoneManager (1 zone)              | 267125/28720 |  7036/  828 |
| BasicZoneManager (all zones)           | 284261/28720 | 24172/  828 |
| BasicZoneManager (all zones+links)     | 292741/28720 | 32652/  828 |
|----------------------------------------+--------------+-------------|
| Basic ZoneSorterByName [1]             | 267749/28728 |   624/    8 |
| Basic ZoneSorterByOffsetAndName [1]    | 267893/28728 |   768/    8 |
|----------------------------------------+--------------+-------------|
| Extended TimeZone (1 zone)             | 269021/29288 |  8932/ 1396 |
| Extended TimeZone (2 zones)            | 269581/29904 |  9492/ 2012 |
| ExtendedZoneManager (1 zone)           | 269165/29296 |  9076/ 1404 |
| ExtendedZoneManager (all zones)        | 301581/29296 | 41492/ 1404 |
| ExtendedZoneManager (all zones+links)  | 311117/29296 | 51028/ 1404 |
|----------------------------------------+--------------+-------------|
| Extended ZoneSorterByName [2]          | 269789/29304 |   624/    8 |
| Extended ZoneSorterByOffsetAndName [2] | 269821/29304 |   656/    8 |
+---------------------------------------------------------------------+

```

## ESP32

* ESP32-01 Dev Board, 240 MHz Tensilica LX6
* Arduino IDE 1.8.19, Arduino CLI 0.31.0
* ESP32 Boards 2.0.7

```
+---------------------------------------------------------------------+
| Functionality                          |  flash/  ram |       delta |
|----------------------------------------+--------------+-------------|
| baseline                               | 228345/21976 |     0/    0 |
|----------------------------------------+--------------+-------------|
| LocalDateTime                          | 230613/21984 |  2268/    8 |
| ZonedDateTime                          | 231761/22000 |  3416/   24 |
| Manual ZoneManager                     | 231769/21976 |  3424/    0 |
|----------------------------------------+--------------+-------------|
| Basic TimeZone (1 zone)                | 235845/22184 |  7500/  208 |
| Basic TimeZone (2 zones)               | 236429/22384 |  8084/  408 |
| BasicZoneManager (1 zone)              | 236009/22200 |  7664/  224 |
| BasicZoneManager (all zones)           | 253113/22200 | 24768/  224 |
| BasicZoneManager (all zones+links)     | 261593/22200 | 33248/  224 |
|----------------------------------------+--------------+-------------|
| Basic ZoneSorterByName [1]             | 236557/22208 |   548/    8 |
| Basic ZoneSorterByOffsetAndName [1]    | 236601/22208 |   592/    8 |
|----------------------------------------+--------------+-------------|
| Extended TimeZone (1 zone)             | 237761/22592 |  9416/  616 |
| Extended TimeZone (2 zones)            | 238329/23208 |  9984/ 1232 |
| ExtendedZoneManager (1 zone)           | 237893/22600 |  9548/  624 |
| ExtendedZoneManager (all zones)        | 270277/22600 | 41932/  624 |
| ExtendedZoneManager (all zones+links)  | 279797/22600 | 51452/  624 |
|----------------------------------------+--------------+-------------|
| Extended ZoneSorterByName [2]          | 238437/22608 |   544/    8 |
| Extended ZoneSorterByOffsetAndName [2] | 238485/22608 |   592/    8 |
+---------------------------------------------------------------------+

```

RAM usage remains constant as more objects are created, which indicates that an
initial pool of a certain minimum size is created regardless of the actual RAM
usage by objects.

## Teensy 3.2

* 96 MHz ARM Cortex-M4
* Arduino IDE 1.8.19, Arduino CLI 0.31.0
* Teensyduino 1.57
* Compiler options: "Faster"

```
+---------------------------------------------------------------------+
| Functionality                          |  flash/  ram |       delta |
|----------------------------------------+--------------+-------------|
| baseline                               |  10060/ 4152 |     0/    0 |
|----------------------------------------+--------------+-------------|
| LocalDateTime                          |  10424/ 4168 |   364/   16 |
| ZonedDateTime                          |  10468/ 4184 |   408/   32 |
| Manual ZoneManager                     |  10196/ 4160 |   136/    8 |
|----------------------------------------+--------------+-------------|
| Basic TimeZone (1 zone)                |  20792/ 4364 | 10732/  212 |
| Basic TimeZone (2 zones)               |  22344/ 4568 | 12284/  416 |
| BasicZoneManager (1 zone)              |  21052/ 4384 | 10992/  232 |
| BasicZoneManager (all zones)           |  38192/ 4384 | 28132/  232 |
| BasicZoneManager (all zones+links)     |  46676/ 4384 | 36616/  232 |
|----------------------------------------+--------------+-------------|
| Basic ZoneSorterByName [1]             |  21564/ 4388 |   512/    4 |
| Basic ZoneSorterByOffsetAndName [1]    |  21628/ 4388 |   576/    4 |
|----------------------------------------+--------------+-------------|
| Extended TimeZone (1 zone)             |  24640/ 4776 | 14580/  624 |
| Extended TimeZone (2 zones)            |  26128/ 5392 | 16068/ 1240 |
| ExtendedZoneManager (1 zone)           |  24836/ 4784 | 14776/  632 |
| ExtendedZoneManager (all zones)        |  57252/ 4784 | 47192/  632 |
| ExtendedZoneManager (all zones+links)  |  66780/ 4784 | 56720/  632 |
|----------------------------------------+--------------+-------------|
| Extended ZoneSorterByName [2]          |  25412/ 4788 |   576/    4 |
| Extended ZoneSorterByOffsetAndName [2] |  25412/ 4788 |   576/    4 |
+---------------------------------------------------------------------+

```

